

__author__    = "Andre Merzky"
__copyright__ = "Copyright 2013, The SAGA Project"
__license__   = "LGPL.v3"



# ------------------------------------------------------------------------------
#
# Atom Types
#
UNKNOWN = 'unknown'
TIME    = 'time'
COMPUTE = 'compute'
COMPUTE = 'compute_asm'
MEMORY  = 'memory' 
STORAGE = 'storage'
NETWORK = 'network'

